==> guikachu-1.3.6.tar.gz: guikachu-1.3.6/src/widgets/resource-combo.cc <==
//$Id: resource-combo.cc,v 1.15 2003/12/25 16:07:12 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001-2003 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "resource-combo.h"

#include "config.h"
#include <libgnome/libgnome.h>

#include <vector>
#include "resource-manager.h"

#include "resource-factory.h"

using namespace Guikachu::GUI::PropertyEditors;

ResourceCombo::ResourceCombo (ResourceManager *manager_,
			      Resources::Type  type,
			      property_t      &property_,
			      op_factory_t    *op_factory_):
    Gtk::HBox (false, 5),
    manager (manager_),
    button_edit (_("Edit...")),
    property (property_),
    op_factory (op_factory_)
{
    types.insert (type);

    create_widgets ();
}

ResourceCombo::ResourceCombo (ResourceManager                 *manager_,
			      const std::set<Resources::Type> &types_,
			      property_t                      &property_,
			      op_factory_t                    *op_factory_):
    Gtk::HBox (false, 5),
    manager (manager_),
    button_edit (_("Edit...")),
    types (types_),
    property (property_),
    op_factory (op_factory_)
{
    create_widgets ();
}



void ResourceCombo::create_widgets ()
{
    combo.set_value_in_list (true, true);
    combo.set_use_arrows (true);
    combo.set_case_sensitive (false);
    combo.get_entry ()->set_editable (false);
    
    std::set<Resource*> resource_set = manager->get_resources ();

    Gtk::ListItem *item = new Gtk::ListItem ("");
    item->set_user_data (0);
    item->show ();
    combo.get_list ()->add (*manage (item));
    for (std::set<Resource*>::iterator i = resource_set.begin ();
	 i != resource_set.end (); i++)
	add_resource ((*i));

    button_edit.clicked.connect (slot (this, &ResourceCombo::edit_cb));
    
    manager->resource_created.connect (slot (this, &ResourceCombo::add_resource));
    manager->resource_removed.connect (slot (this, &ResourceCombo::remove_resource));
    combo.get_entry ()->changed.connect (slot (this, &ResourceCombo::entry_cb));
    
    pack_start (combo);
    pack_start (button_edit);
    underline_widget = &combo;
    
    property.changed.connect (slot (this, &ResourceCombo::update));
    update ();
}

void ResourceCombo::add_resource (Resource *res)
{
    if (types.find (res->get_type ()) == types.end ())
	return;

    update_block = true;

    std::string current = property;
    
    Gtk::ListItem *item = new Gtk::ListItem (res->id);
    item->set_user_data (res);
    item->show ();
    combo.get_list ()->add (*manage (item));

    combo.get_entry ()->set_text (current);

    res->changed.connect (bind (slot (this, &ResourceCombo::resource_changed_cb),
				res));
	
    update_block = false;
}

void ResourceCombo::remove_resource (Resource *res)
{
    if (types.find (res->get_type ()) == types.end ())
	return;
	
    update_block = true;

    Gtk::List::ItemList listitems = combo.get_list ()->items ();
    for (Gtk::List::ItemList::iterator i = listitems.begin ();
	 i != listitems.end (); i++)
    {
	if ((*i)->get_user_data () == res)
	    i = listitems.erase (i);
    }

    update_block = false;
}

void ResourceCombo::update ()
{
    update_block = true;

    combo.get_entry ()->set_text (property);	
    button_edit.set_sensitive (property != "");

    update_block = false;
}

void ResourceCombo::edit_cb ()
{
    Resource *res = property.resolve ();
    g_return_if_fail (res != 0);

    Guikachu::GUI::get_resource_editor (res)->show ();
}

void ResourceCombo::entry_cb ()
{
    if (update_block)
	return;
    
    const std::string new_val = combo.get_entry ()->get_text ();
    if (property == new_val)
	return;
    
    op_factory->push_change (new_val);
    property = new_val;
}

void ResourceCombo::resource_changed_cb (Resource *res)
{
    update_block = true;

    using namespace Gtk::List_Helpers;

    ItemList& itemlist = combo.get_list ()->items ();
    ItemList::iterator i = itemlist.begin ();
    while (i != itemlist.end () && (*i)->get_user_data () != res)
	i++;
    g_return_if_fail (i != itemlist.end ());
    
    (*i)->remove ();
    (*i)->add_label (res->id);
    
    update_block = false;
}

==> guikachu_1.4.0.orig.tar.gz: guikachu-1.4.0/src/widgets/resource-combo.cc <==
//$Id: resource-combo.cc,v 1.16 2004/01/18 10:55:39 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001-2003 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "resource-combo.h"

#include "config.h"
#include <libgnome/libgnome.h>

#include <vector>
#include "resource-manager.h"

#include "resource-factory.h"

using namespace Guikachu::GUI::PropertyEditors;

ResourceCombo::ResourceCombo (Resources::Type  type,
			      property_t      &property_,
			      op_factory_t    *op_factory_):
    Gtk::HBox (false, 5),
    manager (property_.get_manager ()),
    button_edit (_("Edit...")),
    property (property_),
    op_factory (op_factory_)
{
    types.insert (type);

    create_widgets ();
}

ResourceCombo::ResourceCombo (const std::set<Resources::Type> &types_,
			      property_t                      &property_,
			      op_factory_t                    *op_factory_):
    Gtk::HBox (false, 5),
    manager (property_.get_manager ()),
    button_edit (_("Edit...")),
    types (types_),
    property (property_),
    op_factory (op_factory_)
{
    create_widgets ();
}



void ResourceCombo::create_widgets ()
{
    combo.set_value_in_list (true, true);
    combo.set_use_arrows (true);
    combo.set_case_sensitive (false);
    combo.get_entry ()->set_editable (false);
    
    std::set<Resource*> resource_set = manager->get_resources ();

    Gtk::ListItem *item = new Gtk::ListItem ("");
    item->set_user_data (0);
    item->show ();
    combo.get_list ()->add (*manage (item));
    for (std::set<Resource*>::iterator i = resource_set.begin ();
	 i != resource_set.end (); i++)
	add_resource ((*i));

    button_edit.clicked.connect (slot (this, &ResourceCombo::edit_cb));
    
    manager->resource_created.connect (slot (this, &ResourceCombo::add_resource));
    manager->resource_removed.connect (slot (this, &ResourceCombo::remove_resource));
    combo.get_entry ()->changed.connect (slot (this, &ResourceCombo::entry_cb));
    
    pack_start (combo);
    pack_start (button_edit);
    underline_widget = &combo;
    
    property.changed.connect (slot (this, &ResourceCombo::update));
    update ();
}

void ResourceCombo::add_resource (Resource *res)
{
    if (types.find (res->get_type ()) == types.end ())
	return;

    update_block = true;

    std::string current = property;
    
    Gtk::ListItem *item = new Gtk::ListItem (res->id);
    item->set_user_data (res);
    item->show ();
    combo.get_list ()->add (*manage (item));

    combo.get_entry ()->set_text (current);

    res->changed.connect (bind (slot (this, &ResourceCombo::resource_changed_cb),
				res));
	
    update_block = false;
}

void ResourceCombo::remove_resource (Resource *res)
{
    if (types.find (res->get_type ()) == types.end ())
	return;
	
    update_block = true;

    Gtk::List::ItemList listitems = combo.get_list ()->items ();
    for (Gtk::List::ItemList::iterator i = listitems.begin ();
	 i != listitems.end (); i++)
    {
	if ((*i)->get_user_data () == res)
	    i = listitems.erase (i);
    }

    update_block = false;
}

void ResourceCombo::update ()
{
    update_block = true;

    combo.get_entry ()->set_text (property);	
    button_edit.set_sensitive (property != "");

    update_block = false;
}

void ResourceCombo::edit_cb ()
{
    Resource *res = property.resolve ();
    g_return_if_fail (res != 0);

    Guikachu::GUI::get_resource_editor (res)->show ();
}

void ResourceCombo::entry_cb ()
{
    if (update_block)
	return;
    
    const std::string new_val = combo.get_entry ()->get_text ();
    if (property == new_val)
	return;
    
    op_factory->push_change (new_val);
    property = new_val;
}

void ResourceCombo::resource_changed_cb (Resource *res)
{
    update_block = true;

    using namespace Gtk::List_Helpers;

    ItemList& itemlist = combo.get_list ()->items ();
    ItemList::iterator i = itemlist.begin ();
    while (i != itemlist.end () && (*i)->get_user_data () != res)
	i++;
    g_return_if_fail (i != itemlist.end ());
    
    (*i)->remove ();
    (*i)->add_label (res->id);
    
    update_block = false;
}
